#!/usr/bin/env ruby

# == Counter: Count the lines in a file
#
# Take a file and count the number of unique lines then output those lines with their count
# and percentage
#
# == Usage
#
# counter [OPTION] ... FILENAME
#
# -h, --help, -?:
#	show help
#
# --count, -c:
#	do not show the count
#
# --percentage, -p:
#	do not show the percentage
#
# --total, -t:
#	do not show the total
#
# FILENAME: the file to count
#
# Author:: Robin Wood (robin@digininja.org)
# Copyright:: Copyright (c) Robin Wood 2011
# Licence:: GPL
#

require 'getoptlong'

class String
	def is_numeric?
		Integer self rescue false
	end
end

def is_numeric val
	return val.to_s =~ /^[0-9]+$/
end

opts = GetoptLong.new(
	[ '--help', '-h', "-?", GetoptLong::NO_ARGUMENT ],
	[ '--percentage', '-p', GetoptLong::NO_ARGUMENT ],
	[ '--count', '-c', GetoptLong::NO_ARGUMENT ],
	[ '--total', '-t', GetoptLong::NO_ARGUMENT ],
	[ '--lines', "-l" , GetoptLong::REQUIRED_ARGUMENT ],
	[ "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"counter 1.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Usage: counter [OPTION] ... FILENAME
	--help, -h: show help
	--percentage, -p: hide the percentage
	--count, -c: hide the count
	--total, -t: hide the total
	--lines K, -l K: print the first K lines, default is all

	FILENAME: The file to count

"
	exit
end

show_count = true
show_percentage = true
show_total = true
cap_at = nil

begin
	opts.each do |opt, arg|
		case opt
			when '--help'
				usage
			when "--total"
				show_total = false
			when "--percentage"
				show_percentage = false
			when "--count"
				show_count = false
			when "--lines"
				if arg.is_numeric?
					cap_at = arg.to_i
					if cap_at <= 0
						puts"counter 1.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Please enter a positive number of lines

"
						exit 1
					end
				else
					puts"counter 1.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Invalid number of lines

"
	exit 1
				end
		end
	end
rescue => e
	puts e.inspect
	puts
	usage
end

if ARGV.length != 1
	puts"counter 1.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Please specify the file to count

"
	exit 1
end

filename = ARGV.shift

if !File.exist? filename
	puts"counter 1.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Can't find the specified file

"
	exit 2
end

words = {}
total_lines = 0

File.open(filename, "r").each_line { |line|
	total_lines += 1

	line.strip!
	if words.has_key?(line)
		words[line] += 1
	else
		words[line] = 1
	end
}

if cap_at.nil?
	cap_at = words.length
end

# The default is to sort lowest to highest, the -1 just inverts that
words.sort{|a,b| (a[1]<=>b[1]) * -1}[0, cap_at].each { |elem|
	percentage = sprintf("%.2f", (elem[1].to_f / total_lines) * 100)
	if elem[1] == 1
		next
	end
	if show_count and show_percentage
		puts "#{elem[1]} (#{percentage}%): #{elem[0]}"
	elsif show_count
		puts "#{elem[1]}: #{elem[0]}"
	elsif show_percentage
		puts "#{percentage}%: #{elem[0]}"
	else
		puts "#{elem[0]}"
	end
}

if show_total
	puts
	puts "Total entries = " + total_lines.to_s
	uniq_words = words.to_a.uniq
	puts "Total unique entries = " + uniq_words.length.to_s
	uniq_words = Array.new(words.to_a.uniq)

	puts

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^.{1}$/) 
			total += y[1] 
		end
	}
	puts "One character = " + total.to_s

	char_names = {
					2 => "Two",
					3 => "Three",
					4 => "Four",
					5 => "Five",
					6 => "Six",
					7 => "Seven",
					8 => "Eight",
				}

	2.upto(8) do |len|
		total = 0
		uniq_words.each { |y|
			if (y[0] =~ /^.{#{len}}$/) 
				total += y[1] 
			end
		}

		puts char_names[len] + " characters = " + total.to_s
	end

	puts

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^.{1,6}$/) 
			total += y[1] 
		end
	}
	puts "One to six characters = " + total.to_s

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^.{1,8}$/) 
			total += y[1] 
		end
	}
	puts "One to eight characters = " + total.to_s

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^.{9,}$/) 
			total += y[1] 
		end
	}
	puts "More than eight characters = " + total.to_s

	puts

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^[a-z]*$/) 
			total += y[1] 
		end
	}
	puts "Only lowercase alpha = " + total.to_s

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^[A-Z]*$/) 
			total += y[1] 
		end
	}
	puts "Only uppercase alpha = " + total.to_s

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^[a-zA-Z]*$/) 
			total += y[1] 
		end
	}
	puts "Only alpha = " + total.to_s

	total = 0
	uniq_words.each { |y|
		if (y[0] =~ /^[0-9]*$/) 
			total += y[1] 
		end
	}
	puts "Only numeric = " + total.to_s
end
